// Copyright 2022 schukai GmbH
// SPDX-License-Identifier: AGPL-3.0

package configuration

import (
	"github.com/stretchr/testify/assert"
	"os"
	"testing"
)

// Example for README
// Print are commented out because they are only for demonstration
func TestReadmeExample2(t *testing.T) {
	config := struct {
		Host string `env:"HOST"`
	}{
		Host: "localhost",
	}

	// Set value
	os.Setenv("HOST", "www.example.com")
	defer os.Unsetenv("HOST")

	s := New(config)
	//fmt.Println(s.Config().Host) // localhost
	assert.Equal(t, s.Config().Host, "localhost")

	s.InitFromEnv("") // no prefix
	//fmt.Println(s.Config().Host) // www.example.com
	assert.Equal(t, s.Config().Host, "www.example.com")

}

// This test need to source the file env.sh before running the test
func TestEnvFromFile(t *testing.T) {
	config := struct {
		A   int    `env:"A"`
		B   string `env:"B"`
		C   string `env:"C"`
		D   string `env:"D"`
		E   string `env:"E"`
		F   string `env:"F"`
		DSN string `env:"DSN"`
	}{}

	s := New(config)
	s.InitFromEnv("GOTEST_")
	assert.Equal(t, 1, s.Config().A)
	assert.Equal(t, "1", s.Config().B)
	assert.Equal(t, "2", s.Config().C)
	assert.Equal(t, "\"A", s.Config().D)
	assert.Equal(t, "B\"", s.Config().E)
	assert.Equal(t, "F", s.Config().F)
	assert.Equal(t, "user:password@tcp(localhost:3306)/dbname", s.Config().DSN)

}

func TestWithPrefix(t *testing.T) {
	config := struct {
		Host string `env:"HOST"`
	}{
		Host: "localhost",
	}

	// Set value
	os.Setenv("XYZ_HOST", "www.example.com")
	defer os.Unsetenv("XYZ_HOST")

	s := New(config)
	//fmt.Println(s.Config().Host) // localhost
	assert.Equal(t, s.Config().Host, "localhost")

	s.InitFromEnv("XYZ") // no prefix
	//fmt.Println(s.Config().Host) // www.example.com
	assert.Equal(t, s.Config().Host, "www.example.com")

}

func TestFromEnvNotSet(t *testing.T) {
	config := ConfigStruct6{
		IA: 1234,
		IB: "no",
		IC: false,
		ID: true,
	}

	s := New(config)

	// Value not set
	s.InitFromEnv("VALUE_IB")
	defer os.Unsetenv("VALUE_IB")

	if s.Config().IA != 1234 {
		t.Errorf("Expected 1234, got %d", s.Config().IA)
	}

}

func TestFromEnv(t *testing.T) {
	config := ConfigStruct6{
		IA: 1234,
		IB: "no",
		IC: false,
		ID: true,
	}

	s := New(config)

	// Value not set
	s.InitFromEnv("NIX")

	if s.Config().IA != 1234 {
		t.Errorf("Expected 1234, got %d", s.Config().IA)
	}

	// Set value
	os.Setenv("VALUE_IB", "i-am-b")
	defer os.Unsetenv("VALUE_IB")
	s.InitFromEnv("")

	if s.Config().IB != "i-am-b" {
		t.Errorf("Expected i-am-b, got %s", s.Config().IB)
	}

}
